@@ -82,10 +82,15 @@ typedef struct
 	BYTE tag[3];
 } piv_cert_tags_t;
 static const piv_cert_tags_t piv_cert_tags[] = {
-	{ "Certificate for PIV Authentication", "\x5F\xC1\x05" },
-	{ "Certificate for Digital Signature", "\x5F\xC1\x0A" },
-	{ "Certificate for Key Management", "\x5F\xC1\x0B" },
-	{ "Certificate for Card Authentication", "\x5F\xC1\x01" },
+	{ "X.509 Certificate for PIV Authentication", { 0x5F, 0xC1, 0x05 } },
+	{ "X.509 Certificate for Digital Signature", { 0x5F, 0xC1, 0x0A } },
+	{ "X.509 Certificate for Key Management", { 0x5F, 0xC1, 0x0B } },
+	{ "X.509 Certificate for Card Authentication", { 0x5F, 0xC1, 0x01 } },
+
+	{ "Certificate for PIV Authentication", { 0x5F, 0xC1, 0x05 } },
+	{ "Certificate for Digital Signature", { 0x5F, 0xC1, 0x0A } },
+	{ "Certificate for Key Management", { 0x5F, 0xC1, 0x0B } },
+	{ "Certificate for Card Authentication", { 0x5F, 0xC1, 0x01 } },
 };
 
 static const BYTE APDU_PIV_SELECT_AID[] = { 0x00, 0xA4, 0x04, 0x00, 0x09, 0xA0, 0x00, 0x00,
@@ -130,12 +135,17 @@ static SECURITY_STATUS NCryptP11StorageProvider_dtor(NCRYPT_HANDLE handle)
 
 static void fix_padded_string(char* str, size_t maxlen)
 {
-	char* ptr = str + maxlen - 1;
+	if (maxlen == 0)
+		return;
 
-	while (ptr > str && *ptr == ' ')
+	WINPR_ASSERT(str);
+	char* ptr = &str[maxlen - 1];
+
+	while ((ptr > str) && (*ptr == ' '))
+	{
+		*ptr = '\0';
 		ptr--;
-	ptr++;
-	*ptr = 0;
+	}
 }
 
 static BOOL attributes_have_unallocated_buffers(CK_ATTRIBUTE_PTR attributes, CK_ULONG count)
@@ -361,9 +371,8 @@ static void log_(const char* tag, const char* msg, CK_RV rv, CK_ULONG index, CK_
 	if (!WLog_IsLevelActive(log_cached_ptr, log_level))
 		return;
 
-	WLog_PrintMessage(log_cached_ptr, WLOG_MESSAGE_TEXT, log_level, line, file, fkt,
-	                  "%s for slot #%" PRIu32 "(%" PRIu32 "), rv=%s", msg, index, slot,
-	                  CK_RV_error_string(rv));
+	WLog_PrintTextMessage(log_cached_ptr, log_level, line, file, fkt,
+	                      "%s for slot #%lu(%lu), rv=%s", msg, index, slot, CK_RV_error_string(rv));
 }
 
 static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKeysState* state)
@@ -433,39 +442,42 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 			goto cleanup_FindObjectsInit;
 		}
 
-		CK_ULONG nslotObjects = 0;
-		WINPR_ASSERT(p11->C_FindObjects);
-		rv = p11->C_FindObjects(session, &slotObjects[0], ARRAYSIZE(slotObjects), &nslotObjects);
-		if (rv != CKR_OK)
 		{
-			loge(TAG, "unable to findObjects", rv, i, state->slots[i]);
-			goto cleanup_FindObjects;
-		}
-
-		WLog_DBG(TAG, "slot has %d objects", nslotObjects);
-		for (CK_ULONG j = 0; j < nslotObjects; j++)
-		{
-			NCryptKeyEnum* key = &state->keys[state->nKeys];
-			CK_OBJECT_CLASS dataClass = CKO_PUBLIC_KEY;
-			CK_ATTRIBUTE key_or_certAttrs[] = {
-				{ CKA_ID, &key->id, sizeof(key->id) },
-				{ CKA_CLASS, &dataClass, sizeof(dataClass) },
-				{ CKA_LABEL, &key->keyLabel, sizeof(key->keyLabel) },
-				{ CKA_KEY_TYPE, &key->keyType, sizeof(key->keyType) }
-			};
-
-			rv = object_load_attributes(provider, session, slotObjects[j], key_or_certAttrs,
-			                            ARRAYSIZE(key_or_certAttrs));
+			CK_ULONG nslotObjects = 0;
+			WINPR_ASSERT(p11->C_FindObjects);
+			rv =
+			    p11->C_FindObjects(session, &slotObjects[0], ARRAYSIZE(slotObjects), &nslotObjects);
 			if (rv != CKR_OK)
 			{
-				WLog_ERR(TAG, "error getting attributes, rv=%s", CK_RV_error_string(rv));
-				continue;
+				loge(TAG, "unable to findObjects", rv, i, state->slots[i]);
+				goto cleanup_FindObjects;
 			}
 
-			key->idLen = key_or_certAttrs[0].ulValueLen;
-			key->slotId = state->slots[i];
-			key->slotInfo = slotInfo;
-			state->nKeys++;
+			WLog_DBG(TAG, "slot has %d objects", nslotObjects);
+			for (CK_ULONG j = 0; j < nslotObjects; j++)
+			{
+				NCryptKeyEnum* key = &state->keys[state->nKeys];
+				CK_OBJECT_CLASS dataClass = CKO_PUBLIC_KEY;
+				CK_ATTRIBUTE key_or_certAttrs[] = {
+					{ CKA_ID, &key->id, sizeof(key->id) },
+					{ CKA_CLASS, &dataClass, sizeof(dataClass) },
+					{ CKA_LABEL, &key->keyLabel, sizeof(key->keyLabel) },
+					{ CKA_KEY_TYPE, &key->keyType, sizeof(key->keyType) }
+				};
+
+				rv = object_load_attributes(provider, session, slotObjects[j], key_or_certAttrs,
+				                            ARRAYSIZE(key_or_certAttrs));
+				if (rv != CKR_OK)
+				{
+					WLog_ERR(TAG, "error getting attributes, rv=%s", CK_RV_error_string(rv));
+					continue;
+				}
+
+				key->idLen = key_or_certAttrs[0].ulValueLen;
+				key->slotId = state->slots[i];
+				key->slotInfo = slotInfo;
+				state->nKeys++;
+			}
 		}
 
 	cleanup_FindObjects:
@@ -536,7 +548,8 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 			dest[0] = 0;
 		return FALSE;
 	}
-	else
+
+	if (dest)
 	{
 		if (retLen + 1 > len)
 		{
@@ -544,11 +557,8 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 			return FALSE;
 		}
 
-		if (dest)
-		{
-			memcpy(dest, r, sizeof(WCHAR) * retLen);
-			dest[retLen] = 0;
-		}
+		memcpy(dest, r, sizeof(WCHAR) * retLen);
+		dest[retLen] = 0;
 	}
 
 	return TRUE;
@@ -660,7 +670,7 @@ static SECURITY_STATUS parseKeyName(LPCWSTR pszKeyName, CK_SLOT_ID* slotId, CK_B
 
 static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR pszScope,
                                          NCryptKeyName** ppKeyName, PVOID* ppEnumState,
-                                         DWORD dwFlags)
+                                         WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)hProvider;
 	P11EnumKeysState* state = (P11EnumKeysState*)*ppEnumState;
@@ -869,8 +879,8 @@ static SECURITY_STATUS get_piv_container_name(NCryptP11KeyHandle* key, const BYT
 	char container_name[PIV_CONTAINER_NAME_LEN + 1] = { 0 };
 	DWORD buf_len = 0;
 	SECURITY_STATUS ret = NTE_BAD_KEY;
-	WinPrAsn1Decoder dec = { 0 };
-	WinPrAsn1Decoder dec2 = { 0 };
+	WinPrAsn1Decoder dec = { .encoding = WINPR_ASN1_BER, { 0 } };
+	WinPrAsn1Decoder dec2 = { .encoding = WINPR_ASN1_BER, { 0 } };
 	size_t len = 0;
 	BYTE tag = 0;
 	BYTE* p = NULL;
@@ -979,7 +989,8 @@ static SECURITY_STATUS check_for_piv_container_name(NCryptP11KeyHandle* key, BYT
 
 static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
                                                  NCryptKeyGetPropertyEnum property, PBYTE pbOutput,
-                                                 DWORD cbOutput, DWORD* pcbResult, DWORD dwFlags)
+                                                 DWORD cbOutput, DWORD* pcbResult,
+                                                 WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	SECURITY_STATUS ret = NTE_FAIL;
 	CK_RV rv = 0;
@@ -1202,7 +1213,8 @@ static SECURITY_STATUS NCryptP11GetProperty(NCRYPT_HANDLE hObject, NCryptKeyGetP
 }
 
 static SECURITY_STATUS NCryptP11OpenKey(NCRYPT_PROV_HANDLE hProvider, NCRYPT_KEY_HANDLE* phKey,
-                                        LPCWSTR pszKeyName, DWORD dwLegacyKeySpec, DWORD dwFlags)
+                                        LPCWSTR pszKeyName, WINPR_ATTR_UNUSED DWORD dwLegacyKeySpec,
+                                        WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	SECURITY_STATUS ret = 0;
 	CK_SLOT_ID slotId = 0;
@@ -1272,8 +1284,8 @@ fail:
 }
 
 SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
-                                               LPCWSTR pszProviderName, DWORD dwFlags,
-                                               LPCSTR* modulePaths)
+                                               WINPR_ATTR_UNUSED LPCWSTR pszProviderName,
+                                               WINPR_ATTR_UNUSED DWORD dwFlags, LPCSTR* modulePaths)
 {
 	SECURITY_STATUS status = ERROR_INVALID_PARAMETER;
 	LPCSTR defaultPaths[] = { "p11-kit-proxy.so", "opensc-pkcs11.so", NULL };
@@ -1298,16 +1310,18 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 			goto out_load_library;
 		}
 
-		c_get_function_list_t c_get_function_list =
-		    GetProcAddressAs(library, "C_GetFunctionList", c_get_function_list_t);
-
-		if (!c_get_function_list)
 		{
-			status = NTE_PROV_TYPE_ENTRY_BAD;
-			goto out_load_library;
-		}
+			c_get_function_list_t c_get_function_list =
+			    GetProcAddressAs(library, "C_GetFunctionList", c_get_function_list_t);
 
-		status = initialize_pkcs11(library, c_get_function_list, phProvider);
+			if (!c_get_function_list)
+			{
+				status = NTE_PROV_TYPE_ENTRY_BAD;
+				goto out_load_library;
+			}
+
+			status = initialize_pkcs11(library, c_get_function_list, phProvider);
+		}
 		if (status != ERROR_SUCCESS)
 		{
 			status = NTE_PROVIDER_DLL_FAIL;
